/**
 * Simulation orchestrator. Interact with both user and simulator computation.
 */
var Paquito = function () {
	CLOCK = 0; // Global simulation clock
	
	var started = false; // Is the simulation started
	var link = Link();
	var baseHost = $('#host-1'); // Reference model for output box
	var baseChart = $('#chart-host-1'); // Reference model for chart tab
	var hosts = []; // Store host on the network
	
	/**
	 * Initialize simulation
	 */
	var _init = function() {
		$('#startSimulation').click(function () {
			if (!started) {
				started = true;
				
				$('#startSimulation').addClass('disabled');
				_setupSimulation();
				_manageEvents();
			}
			
			return false;
		});
		
		_manageTabs();
		_filterOutput();
	};
	
	/**
	 * Setup the hosts for the simulation
	 */
	var _setupSimulation = function () {
		for (var hostId=1; hostId <= $('#numHost').val(); hostId++) {
			_createHostOutputBox(hostId);
			var host = Host(hostId, link);

			hosts.push(host);
			host.init({
				link: link,
				packetPayload: $('#packetPayload').val(),
				hostDistance: $('#hostDistance').val(),
				errorRate: $('#errorRate').val() / 100,
				bandwith: $('#bandwidth').val() * $('#bandwidthUnit').val(),
			});
			
			_createHostChartBox(hostId);
		}
		
		_setHostsActions();
		link.connectHosts(hosts);
	};

	/**
	 * Create a output box in the UI for a given host
	 * 
	 * @param {Integer} hostId Identifier of the host
	 */
	var _createHostOutputBox = function (hostId) {
		var newHost = $('#host-'+hostId);
		
		if (newHost.length == 0) {
			newHost = baseHost.clone();
			newHost.attr('id', 'host-'+hostId);
			newHost.find('.host-no').text(hostId);
			
			$('article').append(newHost);
		}
	};
	
	/**
	 * Manage statistics tabs
	 */
	var _manageTabs = function () {
		$('ul.tabNavigation a').live('click', function () {
			var tabContainers = $('div.tabContent > div');

			tabContainers.hide();
			tabContainers.filter(this.hash).show();
			$('ul.tabNavigation a').removeClass('selected');
			$(this).addClass('selected');
			
			if ($(this).children().hasClass('host-no') && started) {
				Stats.plot($(this).find('.host-no').filter(':first').text());
			}
			
			return false;
		}).filter(':first').click();
	};

	/**
	 * Randomly assign action to a host. Define:
	 *   - Which type of data to send
	 *   - Amount of data to send
	 *   - Packet receiver
	 */
	var _setHostsActions = function () {
		var actions = ['video', 'audio', 'data'];
		
		for (var hostId=0; hostId < hosts.length; hostId++) {
			var randomAction = actions[Math.floor(Math.random() * actions.length)];
			
			var randomHost = hostId;
			while (randomHost == hostId) {
				randomHost = Math.floor(Math.random() * hosts.length);
			}
			
			hosts[hostId].set('hosts', hosts);
			
			var avgDataSize = $('#avgDataSize').val() * $('#avgDataSizeUnit').val();
			hosts[hostId].setAction(randomAction, randomHost, avgDataSize);
		}
	};
	
	/**
	 * Create a chart tab for a given host.
	 * 
	 * @param {Integer} hostId Identifier of the host
	 */
	var _createHostChartBox = function (hostId) {
		var newHost = $('#chart-host-'+hostId);
		var newLink = $('a[href=#'+baseChart.attr('id')+']').parent().clone();
		
		if (newHost.length == 0) {
			newHost = baseChart.clone();
			newHost.attr('id', 'chart-host-'+hostId);
			newLink.children('a').attr('href', '#'+newHost.attr('id'));
			newLink.children('a').find('.host-no').text(hostId);
			
			$('.tabContent').append(newHost);
			$('.tabNavigation').append(newLink);
		}
	};
	
	/**
	 * Manage output boxes filter
	 */
	var _filterOutput = function () {
		$('#parameters input[type=checkbox]').click(function () {
			$('.outbox li.'+$(this).val()).toggle();
		});
	};
	
	/**
	 * Event loop going thru event on the link
	 */
	var _manageEvents = function () {
		while (link.inUse()) {
			var event = link.pop();
			
			CLOCK = event.clock;
			
			if (event.type == 'trafic') {
				event.object.set('linkIdle', false);
			} else if (event.type == 'packet') {
				link.setIdle();
				event.object.get('destination').receive(event.object);
			} else if (event.type == 'sense') {
				event.object.send();
			}
		}
		
		Stats.statistics();
	};
	
	// Expose public methods
	return {
		init: _init,
	};
	
}();

//General object prototype overload
Array.prototype.remove = function() {
    var what, a=arguments, L=a.length, ax;
    while(L && this.length){
        what= a[--L];
        while((ax=this.indexOf(what)) != -1){
            this.splice(ax, 1);
        }
    }
    return this;
};

String.prototype.replaceAt=function(index, char) {
    return this.substr(0, index) + char + this.substr(index+char.length);
};
