/**
 * Provides statistics on network 
 */
var Stats = (function () {
	var sent = 0; // Sent packets
	var received = 0; // Received packets
	var corrupted = 0; // Corrupted received packets
	var dataPoints = {};
	var networkStats = [];
	
	/**
	 * Crop decimal to a precision
	 * 
	 * @param {Double} num Number to work on
	 * @param {Integer} precision
	 */
	var _roundDecimal = function (num, precision) {
		var result = Math.round(num*Math.pow(10, precision))/Math.pow(10, precision);
		return result;
	};
	
	/**
	 * Compute general network statistics
	 */
	var _statistics = function () {
		var lossRate = (((sent-received)/(sent+received))*100);
		var errorRate = ((corrupted/(sent+received))*100);
		
		networkStats.push({label: 'Sent', data:  sent});
		networkStats.push({label: 'Received', data: (received-corrupted)});
		networkStats.push({label: 'Corrupted', data: corrupted});
		networkStats.push({label: 'Loss', data: (sent-received)});

		$('#frameTransmited').html(sent);
		$('#frameReceived').html(received);
		$('#lossRate').html(_roundDecimal(lossRate, 2));
		$('#numFrameError').html(corrupted);
		$('#frameErrorRate').html(_roundDecimal(errorRate, 2));
		
		_statsPie();
	};
	
	/**
	 * Draw a pie for the general network statistics
	 */
	var _statsPie = function () {
		$.plot($("#networkStatsPie"), networkStats, {
			series: {
	            pie: { 
	                show: true,
	                radius: 1,
	                label: {
	                    show: true,
	                    radius: 1,
	                    formatter: function(label, series){
	                        return '<div style="font-size:8pt;text-align:center;padding:2px;color:white;">'+label+'<br/>'+Math.round(series.percent)+'%</div>';
	                    },
	                    background: { opacity: 0.8 }
	                }
	            }
	        },
	        legend: {
	            show: false
	        }
		});
	};
	
	/**
	 * Mark packet sent by a host
	 * 
	 * @param {Integer} hostId Identifier of the host
	 * @param {Integer} bits Number of bit sent 
	 */
	var _sent = function (hostId, bits) {
		sent++;
		
		if (!dataPoints.hasOwnProperty(hostId)) {
			_initHostPoint(hostId);
		}
		
		dataPoints[hostId].sent.push([CLOCK, bits]);
	};
	
	/**
	 * Mark packet received by a host
	 * 
	 * @param {Integer} hostId Identifier of the host
	 * @param {Integer} bits Number of bit sent 
	 * @param {Boolean} error Packet was corrupted
	 */
	var _received = function (hostId, bits, error) {
		received++;
		
		if (error) {
			corrupted++;
		}
		
		if (!dataPoints.hasOwnProperty(hostId)) {
			_initHostPoint(hostId);
		}
		
		dataPoints[hostId].received.push([CLOCK, bits]);
	};
	
	/**
	 * Initialize points for a given host
	 * 
	 * @param {Integer} hostId Identifier of the host
	 */
	var _initHostPoint = function (hostId) {
		dataPoints[hostId] = { sent: [], received: [] };
	};
	
	/**
	 * Plot sent and received data for each host
	 */
	var _plotData = function (hostId) {
		$.plot($('#chart-host-'+hostId), [
          { label: 'Sent', data: dataPoints[hostId].sent },
          {	label: 'Received', data: dataPoints[hostId].received }
        ],{
			series: {
				lines: { show: true },
				points: { show: true }
			},
			zoom: {
	            interactive: true
	        },
	        pan: {
	            interactive: true
	        }
		});
	};
	
	/**
	 * Display network statistics
	 */
	
	return {
		statistics: _statistics,
		plot: _plotData,
		sent: _sent,
		received: _received,
	};
})();
